# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
from typing import Optional

from azure.core.exceptions import ResourceNotFoundError
from azure.identity import DefaultAzureCredential
from azure.keyvault.secrets import SecretClient
from cached_property import cached_property

from airflow.secrets import BaseSecretsBackend
from airflow.utils.log.logging_mixin import LoggingMixin


class AzureKeyVaultBackend(BaseSecretsBackend, LoggingMixin):
    """
    Retrieves Airflow Connections or Variables from Azure Key Vault secrets.

    The Azure Key Vault can be configured as a secrets backend in the ``airflow.cfg``:

    .. code-block:: ini

        [secrets]
        backend = airflow.providers.microsoft.azure.secrets.azure_key_vault.AzureKeyVaultBackend
        backend_kwargs = {"connections_prefix": "airflow-connections", "vault_url": "<azure_key_vault_uri>"}

    For example, if the secrets prefix is ``airflow-connections-smtp-default``, this would be accessible
    if you provide ``{"connections_prefix": "airflow-connections"}`` and request conn_id ``smtp-default``.
    And if variables prefix is ``airflow-variables-hello``, this would be accessible
    if you provide ``{"variables_prefix": "airflow-variables"}`` and request variable key ``hello``.

    :param connections_prefix: Specifies the prefix of the secret to read to get Connections
    :type connections_prefix: str
    :param variables_prefix: Specifies the prefix of the secret to read to get Variables
    :type variables_prefix: str
    :param config_prefix: Specifies the prefix of the secret to read to get Variables.
    :type config_prefix: str
    :param vault_url: The URL of an Azure Key Vault to use
    :type vault_url: str
    :param sep: separator used to concatenate secret_prefix and secret_id. Default: "-"
    :type sep: str
    """

    def __init__(
        self,
        connections_prefix='airflow-connections',  # type: str
        variables_prefix='airflow-variables',  # type: str
        config_prefix='airflow-config',  # type: str
        vault_url='',  # type: str
        sep='-',  # type: str
        **kwargs
    ):
        super(AzureKeyVaultBackend, self).__init__()
        self.vault_url = vault_url
        self.connections_prefix = connections_prefix.rstrip(sep)
        self.variables_prefix = variables_prefix.rstrip(sep)
        self.config_prefix = config_prefix.rstrip(sep)
        self.sep = sep
        self.kwargs = kwargs

    @cached_property
    def client(self):
        """
        Create a Azure Key Vault client.
        """
        credential = DefaultAzureCredential()
        client = SecretClient(vault_url=self.vault_url, credential=credential, **self.kwargs)
        return client

    def get_conn_uri(self, conn_id):
        # type: (str) -> Optional[str]
        """
        Get an Airflow Connection URI from an Azure Key Vault secret

        :param conn_id: The Airflow connection id to retrieve
        :type conn_id: str
        """
        return self._get_secret(self.connections_prefix, conn_id)

    def get_variable(self, key):
        # type: (str) -> Optional[str]
        """
        Get an Airflow Variable from an Azure Key Vault secret.

        :param key: Variable Key
        :type key: str
        :return: Variable Value
        """
        return self._get_secret(self.variables_prefix, key)

    def get_config(self, key):
        # type: (str) -> Optional[str]
        """
        Get Airflow Configuration

        :param key: Configuration Option Key
        :return: Configuration Option Value
        """
        return self._get_secret(self.config_prefix, key)

    @staticmethod
    def build_path(path_prefix, secret_id, sep='-'):
        # type: (str, str, str) -> str
        """
        Given a path_prefix and secret_id, build a valid secret name for the Azure Key Vault Backend.
        Also replaces underscore in the path with dashes to support easy switching between
        environment variables, so ``connection_default`` becomes ``connection-default``.

        :param path_prefix: The path prefix of the secret to retrieve
        :type path_prefix: str
        :param secret_id: Name of the secret
        :type secret_id: str
        :param sep: Separator used to concatenate path_prefix and secret_id
        :type sep: str
        """
        path = '{}{}{}'.format(path_prefix, sep, secret_id)
        return path.replace('_', sep)

    def _get_secret(self, path_prefix, secret_id):
        # type: (str, str) -> Optional[str]
        """
        Get an Azure Key Vault secret value

        :param path_prefix: Prefix for the Path to get Secret
        :type path_prefix: str
        :param secret_id: Secret Key
        :type secret_id: str
        """
        name = self.build_path(path_prefix, secret_id, self.sep)
        try:
            secret = self.client.get_secret(name=name)
            return secret.value
        except ResourceNotFoundError as ex:
            self.log.debug('Secret %s not found: %s', name, ex)
            return None



# -*- coding: utf-8 -*-
#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
#
"""
Utilities module for cli
"""
from __future__ import absolute_import
from __future__ import print_function

import functools
import getpass
import json
import os
import socket
import struct
import sys
from argparse import Namespace
from datetime import datetime
from fcntl import ioctl
from termios import TIOCGWINSZ

from airflow.models import Log
from airflow.utils import cli_action_loggers
from airflow.utils.platform import is_terminal_support_colors


def action_logging(f):
    """
    Decorates function to execute function at the same time submitting action_logging
    but in CLI context. It will call action logger callbacks twice,
    one for pre-execution and the other one for post-execution.

    Action logger will be called with below keyword parameters:
        sub_command : name of sub-command
        start_datetime : start datetime instance by utc
        end_datetime : end datetime instance by utc
        full_command : full command line arguments
        user : current user
        log : airflow.models.log.Log ORM instance
        dag_id : dag id (optional)
        task_id : task_id (optional)
        execution_date : execution date (optional)
        error : exception instance if there's an exception

    :param f: function instance
    :return: wrapped function
    """
    @functools.wraps(f)
    def wrapper(*args, **kwargs):
        """
        An wrapper for cli functions. It assumes to have Namespace instance
        at 1st positional argument

        :param args: Positional argument. It assumes to have Namespace instance
            at 1st positional argument
        :param kwargs: A passthrough keyword argument
        """
        assert args
        assert isinstance(args[0], Namespace), \
            "1st positional argument should be argparse.Namespace instance, " \
            "but {}".format(args[0])
        metrics = _build_metrics(f.__name__, args[0])
        cli_action_loggers.on_pre_execution(**metrics)
        try:
            return f(*args, **kwargs)
        except Exception as e:
            metrics['error'] = e
            raise
        finally:
            metrics['end_datetime'] = datetime.utcnow()
            cli_action_loggers.on_post_execution(**metrics)

    return wrapper


def _build_metrics(func_name, namespace):
    """
    Builds metrics dict from function args
    It assumes that function arguments is from airflow.bin.cli module's function
    and has Namespace instance where it optionally contains "dag_id", "task_id",
    and "execution_date".

    :param func_name: name of function
    :param namespace: Namespace instance from argparse
    :return: dict with metrics
    """
    sensitive_fields = {'-p', '--password', '--conn-password'}
    full_command = list(sys.argv)
    for idx, command in enumerate(full_command):  # pylint: disable=too-many-nested-blocks
        if command in sensitive_fields:
            # For cases when password is passed as "--password xyz" (with space between key and value)
            full_command[idx + 1] = "*" * 8
        else:
            # For cases when password is passed as "--password=xyz" (with '=' between key and value)
            for sensitive_field in sensitive_fields:
                if command.startswith('{}='.format(sensitive_field)):
                    full_command[idx] = '{}={}'.format(sensitive_field, "*" * 8)

    metrics = {'sub_command': func_name, 'start_datetime': datetime.utcnow(),
               'full_command': '{}'.format(full_command), 'user': getpass.getuser()}

    assert isinstance(namespace, Namespace)
    tmp_dic = vars(namespace)
    metrics['dag_id'] = tmp_dic.get('dag_id')
    metrics['task_id'] = tmp_dic.get('task_id')
    metrics['execution_date'] = tmp_dic.get('execution_date')
    metrics['host_name'] = socket.gethostname()

    extra = json.dumps(dict((k, metrics[k]) for k in ('host_name', 'full_command')))
    log = Log(
        event='cli_{}'.format(func_name),
        task_instance=None,
        owner=metrics['user'],
        extra=extra,
        task_id=metrics.get('task_id'),
        dag_id=metrics.get('dag_id'),
        execution_date=metrics.get('execution_date'))
    metrics['log'] = log
    return metrics


class ColorMode:
    """
    Coloring modes. If `auto` is then automatically detected.
    """
    ON = "on"
    OFF = "off"
    AUTO = "auto"


def should_use_colors(args):
    """
    Processes arguments and decides whether to enable color in output
    """
    if args.color == ColorMode.ON:
        return True
    if args.color == ColorMode.OFF:
        return False
    return is_terminal_support_colors()


def get_terminal_size(fallback=(80, 20)):
    """Return a tuple of (terminal height, terminal width)."""
    try:
        return struct.unpack('hhhh', ioctl(sys.__stdout__, TIOCGWINSZ, '\000' * 8))[0:2]
    except IOError:
        # when the output stream or init descriptor is not a tty, such
        # as when when stdout is piped to another program
        pass
    try:
        return int(os.environ.get('LINES')), int(os.environ.get('COLUMNS'))
    except TypeError:
        return fallback


def header(text, fillchar):
    rows, columns = get_terminal_size()
    print(" {} ".format(text).center(columns, fillchar))


def deprecated_action(func=None, new_name=None, sub_commands=False):
    if not func:
        return functools.partial(deprecated_action, new_name=new_name, sub_commands=sub_commands)

    stream = sys.stderr
    try:
        from pip._vendor import colorama
        WINDOWS = (sys.platform.startswith("win") or
                   (sys.platform == 'cli' and os.name == 'nt'))
        if WINDOWS:
            stream = colorama.AnsiToWin32(sys.stderr)
    except Exception:
        colorama = None

    def should_color():
        # Don't colorize things if we do not have colorama or if told not to
        if not colorama:
            return False

        real_stream = (
            stream if not isinstance(stream, colorama.AnsiToWin32)
            else stream.wrapped
        )

        # If the stream is a tty we should color it
        if hasattr(real_stream, "isatty") and real_stream.isatty():
            return True

        if os.environ.get("TERM") and "color" in os.environ.get("TERM"):
            return True

        # If anything else we should not color it
        return False

    @functools.wraps(func)
    def wrapper(args):
        if getattr(args, 'deprecation_warning', True):
            command = args.subcommand or args.func.__name__
            if sub_commands:
                msg = (
                    "The mode (-l, -d, etc) options to {!r} have been deprecated and removed in Airflow 2.0,"
                    " please use the get/set/list subcommands instead"
                ).format(command)
            else:
                prefix = "The {!r} command is deprecated and removed in Airflow 2.0, please use "
                if isinstance(new_name, list):
                    msg = prefix.format(args.subcommand)
                    new_names = list(map(repr, new_name))
                    msg += "{}, or {}".format(", ".join(new_names[:-1]), new_names[-1])
                    msg += " instead"
                else:
                    msg = (prefix + "{!r} instead").format(command, new_name)

            if should_color():
                msg = "".join([colorama.Fore.YELLOW, msg, colorama.Style.RESET_ALL])
            print(msg, file=sys.stderr)
        func(args)
    return wrapper



# -*- coding: utf-8 -*-
#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
"""Setup.py for the Airflow project."""

import imp
import io
import logging
import os
import sys
import subprocess
import unittest
from os.path import dirname
from textwrap import wrap

from setuptools import setup, find_packages, Command

logger = logging.getLogger(__name__)

# noinspection PyUnresolvedReferences
version = imp.load_source('airflow.version', os.path.join('airflow', 'version.py')).version  # type: ignore

PY3 = sys.version_info[0] == 3
PY38 = PY3 and sys.version_info[1] >= 8

my_dir = dirname(__file__)

if not PY3:
    # noinspection PyShadowingBuiltins
    FileNotFoundError = IOError

# noinspection PyUnboundLocalVariable
try:
    with io.open(os.path.join(my_dir, 'README.md'), encoding='utf-8') as f:
        long_description = f.read()
except FileNotFoundError:
    long_description = ''


def airflow_test_suite():
    """Test suite for Airflow tests"""
    test_loader = unittest.TestLoader()
    test_suite = test_loader.discover(os.path.join(my_dir, 'tests'), pattern='test_*.py')
    return test_suite


class CleanCommand(Command):
    """
    Command to tidy up the project root.
    Registered as cmdclass in setup() so it can be called with ``python setup.py extra_clean``.
    """

    description = "Tidy up the project root"
    user_options = []  # type: ignore

    def initialize_options(self):
        """Set default values for options."""

    def finalize_options(self):
        """Set final values for options."""

    # noinspection PyMethodMayBeStatic
    def run(self):
        """Run command to remove temporary files and directories."""
        os.chdir(my_dir)
        os.system('rm -vrf ./build ./dist ./*.pyc ./*.tgz ./*.egg-info')


class CompileAssets(Command):
    """
    Compile and build the frontend assets using yarn and webpack.
    Registered as cmdclass in setup() so it can be called with ``python setup.py compile_assets``.
    """

    description = "Compile and build the frontend assets"
    user_options = []  # type: ignore

    def initialize_options(self):
        """Set default values for options."""

    def finalize_options(self):
        """Set final values for options."""

    # noinspection PyMethodMayBeStatic
    def run(self):
        """Run a command to compile and build assets."""
        subprocess.check_call('./airflow/www_rbac/compile_assets.sh')


class ListExtras(Command):
    """
    List all available extras
    Registered as cmdclass in setup() so it can be called with ``python setup.py list_extras``.
    """

    description = "List available extras"
    user_options = []  # type: ignore

    def initialize_options(self):
        """Set default values for options."""

    def finalize_options(self):
        """Set final values for options."""

    # noinspection PyMethodMayBeStatic
    def run(self):
        """List extras."""
        print("\n".join(wrap(", ".join(EXTRAS_REQUIREMENTS.keys()), 100)))


def git_version(version_):
    """
    Return a version to identify the state of the underlying git repo. The version will
    indicate whether the head of the current git-backed working directory is tied to a
    release tag or not : it will indicate the former with a 'release:{version}' prefix
    and the latter with a 'dev0' prefix. Following the prefix will be a sha of the current
    branch head. Finally, a "dirty" suffix is appended to indicate that uncommitted
    changes are present.

    :param str version_: Semver version
    :return: Found Airflow version in Git repo
    :rtype: str
    """
    try:
        import git
        try:
            repo = git.Repo(os.path.join(*[my_dir, '.git']))
        except git.NoSuchPathError:
            logger.warning('.git directory not found: Cannot compute the git version')
            return ''
        except git.InvalidGitRepositoryError:
            logger.warning('Invalid .git directory not found: Cannot compute the git version')
            return ''
    except ImportError:
        logger.warning('gitpython not found: Cannot compute the git version.')
        return ''
    if repo:
        sha = repo.head.commit.hexsha
        if repo.is_dirty():
            return '.dev0+{sha}.dirty'.format(sha=sha)
        # commit is clean
        return '.release:{version}+{sha}'.format(version=version_, sha=sha)
    else:
        return 'no_git_version'


def write_version(filename=os.path.join(*[my_dir, "airflow", "git_version"])):
    """
    Write the Semver version + git hash to file, e.g. ".dev0+2f635dc265e78db6708f59f68e8009abb92c1e65".

    :param str filename: Destination file to write
    """
    text = "{}".format(git_version(version))
    with open(filename, 'w') as file:
        file.write(text)


# 'Start dependencies group' and 'Start dependencies group' are mark for ./test/test_order_setup.py
# If you change this mark you should also change ./test/test_order_setup.py function test_main_dependent_group
# Start dependencies group
async_packages = [
    'eventlet>= 0.9.7',
    'gevent>=0.13',
    'greenlet>=0.4.9',
]
atlas = [
    'atlasclient>=0.1.2',
]
aws = [
    'boto3~=1.10',
]
azure_blob_storage = [
    'azure-storage>=0.34.0, <0.37.0',
    'azure-storage-blob<12.0.0;python_version<"3.6"',
    'azure-storage-blob;python_version>="3.6"',
    'azure-storage-common',
]
azure_container_instances = [
    'azure-mgmt-containerinstance>=1.5.0,<2'
]
azure_cosmos = [
    'azure-cosmos>=3.0.1,<4',
]
azure_data_lake = [
    'azure-datalake-store>=0.0.45',
    'azure-mgmt-datalake-store>=0.5.0',
    'azure-mgmt-resource>=2.2.0',
    'cffi<1.14.0;python_version<"3.0"'
]
azure_secrets = [
    'azure-identity>=1.3.1',
    'azure-keyvault>=4.1.0',
]
cassandra = [
    'cassandra-driver>=3.13.0,<3.21.0',
]
celery = [
    'celery~=4.3;python_version>="3.0"',
    'celery==4.3.1;python_version<"3.0"',
    'flower>=0.7.3, <1.0',
    'kombu==4.6.3;python_version<"3.0"',
    'tornado>=4.2.0, <6.0',  # Dep of flower. Pin to a version that works on Py3.5.2
    'vine~=1.3',  # https://stackoverflow.com/questions/32757259/celery-no-module-named-five
]
cgroups = [
    'cgroupspy>=0.1.4',
]
cloudant = [
    'cloudant>=0.5.9,<2.0',
]
crypto = [
    # Cryptography 3.2 for python 2.7 is broken
    # https://github.com/pyca/cryptography/issues/5359#issuecomment-727622403
    # Snowflake requires <3.0
    'cryptography>=0.9.3,<3.0; python_version<"3.0"',
    'cryptography>=0.9.3;python_version>="3.0"',
]
dask = [
    'distributed>=1.17.1, <2',
]
databricks = [
    'requests>=2.20.0, <3',
]
datadog = [
    'datadog>=0.14.0',
]
doc = [
    'sphinx>=2.1.2;python_version>="3.0"',
    'sphinx==1.8.5;python_version<"3.0"',
    'sphinx-argparse>=0.1.13',
    'sphinx-autoapi==1.0.0',
    'sphinx-copybutton;python_version>="3.6"',
    'sphinx-jinja~=1.1',
    'sphinx-rtd-theme>=0.1.6',
    'sphinxcontrib-httpdomain>=1.7.0',
]
docker = [
    'docker~=3.0',
]
druid = [
    'pydruid>=0.4.1,<=0.5.8',
]
elasticsearch = [
    'elasticsearch>=5.0.0,<6.0.0',
    'elasticsearch-dsl>=5.0.0,<6.0.0',
]
flask_oauth = [
    'Flask-OAuthlib>=0.9.1,<0.9.6',  # Flask OAuthLib 0.9.6 requires Flask-Login 0.5.0 - breaks FAB
    'oauthlib!=2.0.3,!=2.0.4,!=2.0.5,<3.0.0,>=1.1.2',
    'requests-oauthlib==1.1.0',
]
gcp = [
    'PyOpenSSL<20.0.0;python_version<"3.0"',
    'PyOpenSSL;python_version>="3.0"',
    'google-api-python-client>=1.6.0, <2.0.0',
    'google-auth>=1.0.0, <2.0.0',
    'google-auth-httplib2>=0.0.1',
    'google-cloud-bigtable>=1.0.0,<2.0.0',
    'google-cloud-container>=0.1.1,<2.0.0',
    'google-cloud-dlp>=0.11.0,<2.0.0',
    'google-cloud-language>=1.1.1,<2.0.0',
    'google-cloud-secret-manager>=0.2.0,<2.0.0',
    'google-cloud-spanner>=1.10.0,<2.0.0',
    'google-cloud-speech>=0.36.3,<2.0.0',
    'google-cloud-storage>=1.16,<2.0.0',
    'google-cloud-texttospeech>=0.4.0,<2',
    'google-cloud-translate>=1.3.3,<2.0.0',
    'google-cloud-videointelligence>=1.7.0,<2.0.0',
    'google-cloud-vision>=0.35.2,<2.0.0',
    'grpcio-gcp>=0.2.2',
    'pandas-gbq',
]
grpc = [
    'grpcio>=1.15.0',
]
hashicorp = [
    'hvac~=0.10',
]
hdfs = [
    'snakebite>=2.7.8;python_version<"3.0"',
    'snakebite-py3;python_version>="3.0"'
]
hive = [
    'hmsclient>=0.1.0',
    'pyhive[hive]>=0.6.0',
]
jdbc = [
    'JPype1==0.7.1',
    'jaydebeapi>=1.1.1',
]
jenkins = [
    'python-jenkins>=1.0.0',
]
jira = [
    'JIRA>1.0.7',
]
kerberos = [
    'pykerberos>=1.1.13',
    'requests_kerberos>=0.10.0',
    'thrift_sasl>=0.2.0,<0.4.1;python_version<"3.0"',
    'thrift_sasl>=0.2.0;python_version>="3.0"',
]
kubernetes = [
    'cryptography>=2.0.0',
    'kubernetes>=3.0.0, <12.0.0',
]
ldap = [
    'ldap3>=2.5.1',
]
mongo = [
    'dnspython>=1.13.0,<2.0.0',
    'pymongo>=3.6.0,<3.11.0',
]
mssql = [
    'pymssql~=2.1.1',
]
mysql = [
    'mysqlclient>=1.3.6,<1.4',
]
oracle = [
    'cx_Oracle>=5.1.2, <8.0;python_version<"3.0"',
    'cx_Oracle>=5.1.2;python_version>="3.0"',
]
pagerduty = [
    'pypd>=1.1.0',
]
papermill = [
    'papermill[all]>=1.0.0',
    'nteract-scrapbook[all]>=0.2.1',
    'pyarrow<1.0.0',
    'fsspec<0.8.0;python_version=="3.5"',
    'black==20.8b0;python_version>="3.6"'  # we need to limit black version as we have click < 7
]
password = [
    'bcrypt>=2.0.0',
    'flask-bcrypt>=0.7.1',
]
pinot = [
    'pinotdb==0.1.1',
]
postgres = [
    'psycopg2-binary>=2.7.4',
]
presto = [
    'presto-python-client>=0.7.0,<0.8'
]
qds = [
    'qds-sdk>=1.10.4',
]
rabbitmq = [
    'amqp<5.0.0',
]
redis = [
    'redis~=3.2',
]
salesforce = [
    'simple-salesforce>=0.72,<1.0.0',
]
samba = [
    'pysmbclient>=0.1.3',
]
segment = [
    'analytics-python>=1.2.9',
]
sendgrid = [
    'sendgrid>=5.2.0,<6',
]
sentry = [
    'blinker>=1.1',
    'sentry-sdk>=0.8.0',
]
slack = [
    'slackclient>=1.0.0,<2.0.0',
    'websocket-client<0.55.0'
]
snowflake = [
    'boto3<1.11',
    'snowflake-connector-python>=1.5.2',
    'snowflake-sqlalchemy>=1.1.0',
]
ssh = [
    'paramiko>=2.1.1',
    'pysftp>=0.2.9',
    'sshtunnel>=0.1.4,<0.2',
]
statsd = [
    'statsd>=3.3.0, <4.0',
]
vertica = [
    'vertica-python>=0.5.1',
]
virtualenv = [
    'virtualenv',
]
webhdfs = [
    'hdfs[avro,dataframe,kerberos]>=2.0.4',
]
winrm = [
    'pywinrm~=0.4',
]
zendesk = [
    'zdesk',
]
# End dependencies group

all_dbs = (cassandra + cloudant + druid + hdfs + hive + mongo + mssql + mysql +
           pinot + postgres + presto + vertica)

############################################################################################################
# IMPORTANT NOTE!!!!!!!!!!!!!!!
# IF you are removing dependencies from this list, please make sure that you also increase
# DEPENDENCIES_EPOCH_NUMBER in the Dockerfile.ci
############################################################################################################
devel = [
    'beautifulsoup4~=4.7.1',
    'click==6.7',
    'contextdecorator;python_version<"3.4"',
    'coverage',
    'docutils>=0.14, <0.16',
    'ecdsa<0.15',  # Required for moto 1.3.14
    'flake8>=3.6.0',
    'flake8-colors',
    'flaky',
    'freezegun',
    'gitpython',
    'idna<2.9',  # Required for moto 1.3.14
    'importlib-metadata~=2.0; python_version<"3.9"',
    'ipdb',
    'jira',
    'mock;python_version<"3.3"',
    'mongomock',
    'moto==1.3.14',  # TODO - fix Datasync issues to get higher version of moto:
                     #        See: https://github.com/apache/airflow/issues/10985
    'packaging',
    'parameterized',
    'paramiko',
    'pipdeptree',
    'pre-commit',
    'pyrsistent<=0.16.0;python_version<"3.0"',
    'pyrsistent;python_version>="3.0"',
    'pysftp',
    'pytest<6.0.0',  # FIXME: pylint complaining for pytest.mark.* on v6.0
    'pytest-cov',
    'pytest-instafail',
    'pytest-timeouts',
    'pywinrm',
    'qds-sdk>=1.9.6',
    'requests_mock',
    'yamllint',
]
############################################################################################################
# IMPORTANT NOTE!!!!!!!!!!!!!!!
# IF you are removing dependencies from the above list, please make sure that you also increase
# DEPENDENCIES_EPOCH_NUMBER in the Dockerfile.ci
############################################################################################################

if PY3:
    devel += ['mypy==0.720']
else:
    devel += ['unittest2']

devel_minreq = aws + cgroups + devel + doc + kubernetes + mysql + password
devel_hadoop = devel_minreq + hdfs + hive + kerberos + presto + webhdfs

devel_azure = azure_blob_storage + azure_container_instances + azure_cosmos + azure_data_lake + azure_secrets + devel_minreq  # noqa
devel_all = (all_dbs + atlas + aws +
             devel_azure +
             celery + cgroups + crypto + datadog + devel + doc + docker +
             elasticsearch + gcp + grpc + hashicorp + jdbc + jenkins + kerberos + kubernetes + ldap +
             oracle + papermill + password +
             rabbitmq + redis + samba + segment + sendgrid + sentry + slack + snowflake + ssh +
             virtualenv + webhdfs + zendesk)

# Snakebite is not Python 3 compatible :'(
if PY3:
    package_to_excludes = ['snakebite>=2.7.8', 'snakebite[kerberos]>=2.7.8']
    if PY38:
        package_to_excludes.extend(['pymssql~=2.1.1'])
    devel_all = [package for package in devel_all if package not in package_to_excludes]
    devel_ci = devel_all
else:
    devel_ci = devel_all + ['unittest2']


#####################################################################################################
# IMPORTANT NOTE!!!!!!!!!!!!!!!
# IF you are removing dependencies from this list, please make sure that you also increase
# DEPENDENCIES_EPOCH_NUMBER in the Dockerfile
#####################################################################################################
EXTRAS_REQUIREMENTS = {
    'all': devel_all,
    'all_dbs': all_dbs,
    'amazon': aws,
    'apache.atlas': atlas,
    "apache.cassandra": cassandra,
    "apache.druid": druid,
    "apache.hdfs": hdfs,
    "apache.hive": hive,
    "apache.pinot": pinot,
    "apache.presto": presto,
    "apache.webhdfs": webhdfs,
    'async': async_packages,
    'atlas': atlas,
    'aws': aws,
    'azure': azure_blob_storage + azure_container_instances + azure_cosmos + azure_data_lake + azure_secrets,
    'azure_blob_storage': azure_blob_storage,
    'azure_container_instances': azure_container_instances,
    'azure_cosmos': azure_cosmos,
    'azure_data_lake': azure_data_lake,
    'azure_secrets': azure_secrets,
    'cassandra': cassandra,
    'celery': celery,
    'cgroups': cgroups,
    'cloudant': cloudant,
    'cncf.kubernetes': kubernetes,
    'crypto': crypto,
    'dask': dask,
    'databricks': databricks,
    'datadog': datadog,
    'devel': devel_minreq,
    'devel_all': devel_all,
    'devel_azure': devel_azure,
    'devel_ci': devel_ci,
    'devel_hadoop': devel_hadoop,
    'doc': doc,
    'docker': docker,
    'druid': druid,
    'elasticsearch': elasticsearch,
    'emr': aws,
    'gcp': gcp,
    'gcp_api': gcp,
    'github_enterprise': flask_oauth,
    'google': gcp,
    'google_auth': flask_oauth,
    'grpc': grpc,
    'hashicorp': hashicorp,
    'hdfs': hdfs,
    'hive': hive,
    'jdbc': jdbc,
    'jira': jira,
    'kerberos': kerberos,
    'kubernetes': kubernetes,
    'ldap': ldap,
    'mongo': mongo,
    'mssql': mssql,
    'microsoft.azure':
    azure_blob_storage + azure_container_instances + azure_cosmos + azure_data_lake + azure_secrets,
    'microsoft.mssql': mssql,
    'microsoft.winrm': winrm,
    'mysql': mysql,
    'oracle': oracle,
    'papermill': papermill,
    'password': password,
    'pinot': pinot,
    'postgres': postgres,
    'presto': presto,
    'qds': qds,
    'rabbitmq': rabbitmq,
    'redis': redis,
    's3': aws,
    'salesforce': salesforce,
    'samba': samba,
    'segment': segment,
    'sendgrid': sendgrid,
    'sentry': sentry,
    'slack': slack,
    'snowflake': snowflake,
    'ssh': ssh,
    'statsd': statsd,
    'vertica': vertica,
    'virtualenv': virtualenv,
    'webhdfs': webhdfs,
    'winrm': winrm
}

#####################################################################################################
# IMPORTANT NOTE!!!!!!!!!!!!!!!
# IF you are removing dependencies from this list, please make sure that you also increase
# DEPENDENCIES_EPOCH_NUMBER in the Dockerfile.ci
#####################################################################################################
INSTALL_REQUIREMENTS = [
    'alembic>=1.0, <2.0',
    'argcomplete~=1.10',
    'attrs>=20.0, <21.0',
    'cached_property~=1.5',
    # cattrs >= 1.1.0 dropped support for Python 3.6
    'cattrs>=1.0, <1.1.0;python_version<="3.6"',
    'cattrs>=1.0, <2.0;python_version>"3.6"',
    'click<8.0.0;python_version<"3.0"',  # click >8 is python 3.6 only but not marked as such yet
    'colorlog==4.0.2',
    'configparser>=3.5.0, <3.6.0',
    'croniter>=0.3.17, <0.4',
    'cryptography>=0.9.3,<3.0; python_version<"3.0"',  # required by snowflake
    'cryptography>=0.9.3;python_version>="3.0"',
    'dill>=0.2.2, <0.4',
    'email-validator',
    'enum34~=1.1.6;python_version<"3.4"',
    'flask>=1.1.0, <2.0',
    'flask-admin==1.5.4',
    'flask-appbuilder>=1.12.2, <2.0.0;python_version<"3.6"',
    'flask-appbuilder~=2.2;python_version>="3.6"',
    'flask-caching>=1.3.3, <1.4.0',
    'flask-login>=0.3, <0.5',
    'flask-swagger>=0.2.13, <0.3',
    'flask-wtf>=0.14.2, <0.15',
    'funcsigs>=1.0.0, <2.0.0',
    'future>=0.16.0, <0.19',
    'graphviz>=0.12',
    'gunicorn>=19.5.0, <21.0',
    'importlib-metadata~=2.0; python_version<"3.9"',
    'importlib_resources~=1.4',
    'iso8601>=0.1.12',
    'jinja2>=2.10.1, <2.12.0',
    'json-merge-patch==0.2',
    'jsonschema~=3.0',
    'lazy_object_proxy<1.5.0',  # Required to keep pip-check happy with astroid
    'markdown>=2.5.2, <3.0',
    'marshmallow-sqlalchemy>=0.16.1, <0.24.0;python_version>="3.6"',
    'marshmallow-sqlalchemy>=0.16.1, <0.19.0;python_version<"3.6"',
    'packaging',
    'pandas>=0.17.1, <2.0',
    'pendulum==1.4.4',
    'pep562~=1.0;python_version<"3.7"',
    'psutil>=4.2.0, <6.0.0',
    'pygments>=2.0.1, <3.0',
    'python-daemon>=2.1.1',
    'python-dateutil>=2.3, <3',
    'python-nvd3~=0.15.0',
    'python-slugify>=3.0.0,<5.0',
    'requests>=2.20.0, <2.23.0;python_version<"3.0"',  # Required to keep snowflake happy
    'requests>=2.20.0, <2.24.0;python_version>="3.0"',  # Required to keep snowflake happy
    'setproctitle>=1.1.8, <2',
    'sqlalchemy~=1.3',
    'sqlalchemy_jsonfield==0.8.0;python_version<"3.5"',
    'sqlalchemy_jsonfield~=0.9;python_version>="3.5"',
    'tabulate>=0.7.5, <0.9',
    'tenacity==4.12.0',
    'thrift>=0.11.0',
    'typing;python_version<"3.5"',
    'typing-extensions>=3.7.4;python_version<"3.8"',
    'tzlocal>=1.4,<2.0.0',
    'unicodecsv>=0.14.1',
    'werkzeug<1.0.0',
    'zope.deprecation>=4.0, <5.0',
]


def get_dependency_name(dep):
    """Get name of a dependency."""
    return dep.replace(">", '=').replace("<", "=").split("=")[0]


def do_setup():
    """Perform the Airflow package setup."""
    write_version()
    setup(
        name='apache-airflow',
        description='Programmatically author, schedule and monitor data pipelines',
        long_description=long_description,
        long_description_content_type='text/markdown',
        license='Apache License 2.0',
        version=version,
        packages=find_packages(exclude=['tests*', 'airflow.upgrade*']),
        package_data={
            '': ['airflow/alembic.ini', "airflow/git_version", "*.ipynb",
                 "airflow/providers/cncf/kubernetes/example_dags/*.yaml"],
            'airflow.serialization': ["*.json"],
        },
        include_package_data=True,
        zip_safe=False,
        scripts=['airflow/bin/airflow'],
        install_requires=INSTALL_REQUIREMENTS,
        setup_requires=[
            'bowler',
            'docutils>=0.14,<0.16',
            'gitpython>=2.0.2',
            'setuptools',
            'wheel',
        ],
        extras_require=EXTRAS_REQUIREMENTS,
        classifiers=[
            'Development Status :: 5 - Production/Stable',
            'Environment :: Console',
            'Environment :: Web Environment',
            'Intended Audience :: Developers',
            'Intended Audience :: System Administrators',
            'License :: OSI Approved :: Apache Software License',
            'Programming Language :: Python :: 2.7',
            'Programming Language :: Python :: 3.5',
            'Programming Language :: Python :: 3.6',
            'Programming Language :: Python :: 3.7',
            'Programming Language :: Python :: 3.8',
            'Topic :: System :: Monitoring',
        ],
        author='Apache Software Foundation',
        author_email='dev@airflow.apache.org',
        url='http://airflow.apache.org/',
        download_url=(
            'https://archive.apache.org/dist/airflow/' + version),
        cmdclass={
            'extra_clean': CleanCommand,
            'compile_assets': CompileAssets,
            'list_extras': ListExtras,
        },
        test_suite='setup.airflow_test_suite',
        python_requires='>=2.7,!=3.0.*,!=3.1.*,!=3.2.*,!=3.3.*,!=3.4.*',
        project_urls={
            'Documentation': 'https://airflow.apache.org/docs/',
            'Bug Tracker': 'https://github.com/apache/airflow/issues',
            'Source Code': 'https://github.com/apache/airflow',
        },
    )


if __name__ == "__main__":
    # Warn about py2 support going away. This likely won't show up if installed
    # via pip, but we may as well have it here
    if sys.version_info[0] == 2:
        sys.stderr.writelines(
            "DEPRECATION: Python 2.7 will reach the end of its life on January 1st, 2020. Airflow 1.10 "
            "will be the last release series to support Python 2\n"
        )
    do_setup()



# -*- coding: utf-8 -*-
#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.

import time
from airflow.hooks.base_hook import BaseHook
from airflow.exceptions import AirflowException
from datadog import initialize, api

from airflow.utils.log.logging_mixin import LoggingMixin


class DatadogHook(BaseHook, LoggingMixin):
    """
    Uses datadog API to send metrics of practically anything measurable,
    so it's possible to track # of db records inserted/deleted, records read
    from file and many other useful metrics.

    Depends on the datadog API, which has to be deployed on the same server where
    Airflow runs.

    :param datadog_conn_id: The connection to datadog, containing metadata for api keys.
    :param datadog_conn_id: str
    """
    def __init__(self, datadog_conn_id='datadog_default'):
        conn = self.get_connection(datadog_conn_id)
        self.api_key = conn.extra_dejson.get('api_key', None)
        self.app_key = conn.extra_dejson.get('app_key', None)
        self.source_type_name = conn.extra_dejson.get('source_type_name', None)

        # If the host is populated, it will use that hostname instead.
        # for all metric submissions.
        self.host = conn.host

        if self.api_key is None:
            raise AirflowException("api_key must be specified in the "
                                   "Datadog connection details")

        self.log.info("Setting up api keys for Datadog")
        initialize(api_key=self.api_key, app_key=self.app_key)

    def validate_response(self, response):
        if response['status'] != 'ok':
            self.log.error("Datadog returned: %s", response)
            raise AirflowException("Error status received from Datadog")

    def send_metric(self, metric_name, datapoint, tags=None, type_=None, interval=None):
        """
        Sends a single datapoint metric to DataDog

        :param metric_name: The name of the metric
        :type metric_name: str
        :param datapoint: A single integer or float related to the metric
        :type datapoint: int or float
        :param tags: A list of tags associated with the metric
        :type tags: list
        :param type_: Type of your metric: gauge, rate, or count
        :type type_: str
        :param interval: If the type of the metric is rate or count, define the corresponding interval
        :type interval: int
        """
        response = api.Metric.send(
            metric=metric_name,
            points=datapoint,
            host=self.host,
            tags=tags,
            type=type_,
            interval=interval)

        self.validate_response(response)
        return response

    def query_metric(self,
                     query,
                     from_seconds_ago,
                     to_seconds_ago):
        """
        Queries datadog for a specific metric, potentially with some
        function applied to it and returns the results.

        :param query: The datadog query to execute (see datadog docs)
        :type query: str
        :param from_seconds_ago: How many seconds ago to start querying for.
        :type from_seconds_ago: int
        :param to_seconds_ago: Up to how many seconds ago to query for.
        :type to_seconds_ago: int
        """
        now = int(time.time())

        response = api.Metric.query(
            start=now - from_seconds_ago,
            end=now - to_seconds_ago,
            query=query)

        self.validate_response(response)
        return response

    def post_event(self, title, text, aggregation_key=None, alert_type=None, date_happened=None,
                   handle=None, priority=None, related_event_id=None, tags=None, device_name=None):
        """
        Posts an event to datadog (processing finished, potentially alerts, other issues)
        Think about this as a means to maintain persistence of alerts, rather than
        alerting itself.

        :param title: The title of the event
        :type title: str
        :param text: The body of the event (more information)
        :type text: str
        :param aggregation_key: Key that can be used to aggregate this event in a stream
        :type aggregation_key: str
        :param alert_type: The alert type for the event, one of
            ["error", "warning", "info", "success"]
        :type alert_type: str
        :param date_happened: POSIX timestamp of the event; defaults to now
        :type date_happened: int
        :handle: User to post the event as; defaults to owner of the application key used
            to submit.
        :param handle: str
        :param priority: Priority to post the event as. ("normal" or "low", defaults to "normal")
        :type priority: str
        :param related_event_id: Post event as a child of the given event
        :type related_event_id: id
        :param tags: List of tags to apply to the event
        :type tags: list[str]
        :param device_name: device_name to post the event with
        :type device_name: list
        """
        response = api.Event.create(
            title=title,
            text=text,
            aggregation_key=aggregation_key,
            alert_type=alert_type,
            date_happened=date_happened,
            handle=handle,
            priority=priority,
            related_event_id=related_event_id,
            tags=tags,
            host=self.host,
            device_name=device_name,
            source_type_name=self.source_type_name)

        self.validate_response(response)
        return response
